function [loglik,weight1,weight2] = filter_apf(sysm,dim,data,N)
% Evaluating log-likelihood function by auxiliary particle filtering
% Linearizing measurement equation around E(x_t|I_{t-1},y_t) (this expectation
% is approximated by CDKF updating)

global ix1 ix2 
nx = dim.x;
ix1 = 1:dim.x1;
ix2 = dim.x1+1:dim.x;

R2 = sysm.R*sysm.R';    % covariance of structural shocks
H2 = sysm.H*sysm.H';    % covariance of measurement errors

% initials
T = size(data,1);
loglik = 1:T;

h = sqrt(3);

% initial value of s at t=0
Sigma = solve_cov(sysm,dim);
Sxf = zeros(nx,nx);
Sxf(ix2,ix2) = triag(chol(Sigma)');

xt = zeros(nx,N);
xt(ix2,:) = chol(Sigma)'*randn(dim.u,N);
Exf = zeros(nx,1);

Sxp = zeros(nx,nx);
Sxp(ix2,ix2) = triag(chol(R2(ix2,ix2))');

lnwt2 = log(ones(N,1));
max2 = max(lnwt2);
wt2 = exp(lnwt2);

%ess1 = 1:T;
%ess2 = 1:T;
weight1 = nan(N,T);
weight2 = nan(N,T);
for t = 1:T
       
    yt = data(t,:)';
    x0 = cdkf_upda(sysm,dim,Exf,Sxf,h,yt);  % Linearization location
    
    Exp = fun_h(sysm,xt);    % E(x_t|x_{t-1})  
    Dg = gre_g(sysm,dim,x0);
    aux1 = fun_g(sysm,x0) - Dg*x0;   % auxiliary variable 1
    Eyp = aux1(:,ones(1,N)) + Dg*Exp;        % E(y_t|x_{t-1})
    
    A = triag([Sxp,zeros(nx,dim.e);Dg*Sxp,sysm.H]);
    Sx = A(1:nx,1:nx);
    K = A(1:nx,nx+1:end);
    Sy = A(nx+1:end,nx+1:end);
    K = K*Sy'/(Sy*Sy');
    
    ep = yt(:,ones(N,1)) - Eyp; 
    Pyp = Sy*Sy';
    
    lpl = lmnpdf2(ep,Pyp);     
    lnwt1 = lnwt2 + lpl;
    max1 = max(lnwt1);
    wt1 = exp(lnwt1-max1);
    [~,ind] = resmpl_mul(xt,wt1/sum(wt1));
    lncon = log(sum(wt1)) - log(sum(wt2)) + max1 - max2;
    %cv = sum((wt1-mean(wt1)).^2)/(N-1)/mean(wt1)^2;
    %ess1(t) = N/(1+cv);   
    weight1(:,t) = wt1; 

    Ex = Exp(:,ind) + K*ep(:,ind);
    Px = Sx*Sx';
    
    % sampling
    e3 = Sx(ix2,ix2)*randn(dim.u,N);  %Ex = Exp for ix1
    xt(ix1,:) = Ex(ix1,:);
    xt(ix2,:) = Ex(ix2,:) + e3;
           
    % evaluates importance weights
    e1 = xt(ix2,:) - Exp(ix2,ind);
    e2 = yt(:,ones(N,1)) - fun_g(sysm,xt);
    lnwt2 = lmnpdf2(e1,R2(ix2,ix2)) + lmnpdf2(e2,H2) - lmnpdf2(e3,Px(ix2,ix2)) - lpl(ind);
    max2 = max(lnwt2);
    wt2 = exp(lnwt2-max2);
    loglik(t) = log(sum(wt2)) - log(N) + max2 + lncon;
    
    weight2(:,t) = wt2;   
   % contribution to loglikelilhood
    %cv = sum((wt2-mean(wt2)).^2)/(N-1)/mean(wt2)^2;
    %ess2(t) = N/(1+cv);
    % resampling
    Exf = xt*wt2/sum(wt2);
    Sxf = triag(sqrt(wt2(:,ones(1,nx))/sum(wt2))'.*(xt - Exf(:,ones(1,N))));
    
end
loglik = sum(loglik);
end


%--------------------------------------------------------------------------
function Ex = cdkf_upda(sysm,dim,Exf,Sxf,h,y)
nx = dim.x;

Sw = sysm.R;
Sv = sysm.H;

S1xx = fun_h2(sysm,Exf(:,ones(1,nx)),h*Sxf)/(2*h);
S2xx = fun_h3(sysm,h*Sxf)*(sqrt(h^2-1))/(2*h^2);

Exp = (h^2-nx)/(h^2)*fun_h(sysm,Exf) + sum(fun_h1(sysm,Exf,h*Sxf),2)/(2*h^2);
Sxp = triag([S1xx,Sw,S2xx]);
Eyp = (h^2-nx)/(h^2)*(fun_g(sysm,Exp)) + sum(fun_g1(sysm,Exp,h*Sxp),2)/(2*h^2);
    
% filtering
S1yx = fun_g2(sysm,Exp(:,ones(1,nx)),h*Sxp)/(2*h);
S2yx = fun_g3(sysm,h*Sxp)*(sqrt(h^2-1))/(2*h^2);

Syp = triag([S1yx,Sv,S2yx]);
Pyp = Syp*Syp';
K = Sxp*S1yx'/Pyp;
    
et = y - Eyp;
Ex = Exp + K*et;
end

function y = fun_g(sysm,x)
N = size(x,2);
z = sysm.gc(:,ones(N,1)) + sysm.gl*x + qf(sysm.gq,x); 
y = sysm.c(:,ones(N,1)) + sysm.zz*z + sysm.zx*x;
end

function y = fun_g1(sysm,x,s)
N = size(s,2);
z = sysm.gc + sysm.gl*x + qf(sysm.gq,x);
z = z(:,ones(N,1)) + qf(sysm.gq,s);
y = sysm.c + sysm.zx*x;
y = 2*(y(:,ones(N,1)) + sysm.zz*z);
end

function y = fun_g2(sysm,x,s)
z = sysm.gl*s + qf(sysm.gq,x,s) + qf(sysm.gq,s,x);
y = 2*(sysm.zz*z + sysm.zx*s);
end

function y = fun_g3(sysm,s)
y = 2*sysm.zz*qf(sysm.gq,s);
end

function y = fun_h(sysm,x)
global ix1
N = size(x,2);
y = sysm.hc(:,ones(N,1)) + sysm.hl*x;
y(ix1,:) = y(ix1,:) + qf(sysm.hq,x);
end

function y = fun_h1(sysm,x,s)
global ix1
N = size(s,2);
y = sysm.hc + sysm.hl*x;
y = 2*y(:,ones(N,1));
d = qf(sysm.hq,x);
y(ix1,:) = y(ix1,:) + 2*(d(:,ones(N,1))+qf(sysm.hq,s));  
end

function y = fun_h2(sysm,x,s)
global ix1
y = 2*sysm.hl*s;
y(ix1,:) = y(ix1,:) + 2*(qf(sysm.hq,x,s)+qf(sysm.hq,s,x));  
end

function y = fun_h3(sysm,s)
global ix1
[nx,N] = size(s);
y = zeros(nx,N);
y(ix1,:) = 2*qf(sysm.hq,s);
end

function y = gre_g(sysm,dim,x)
nx = dim.x;
nz = dim.z;
N = size(x,2);
y = sysm.zz*sysm.gl + sysm.zx;
a = sysm.gq*x;
b = reshape(a,nx,nz*N);
y = y + 2*sysm.zz*b'; 
end

function y = qf(Q,x,s)
if nargin == 2
    s = x;
end
[nx,N] = size(x);
ny = size(Q,1)/nx;
a = Q*x;
b = a.*repmat(s,ny,1);
c = sum(reshape(b,nx,ny*N),1);
y = reshape(c,ny,N);
end

function[G0] = solve_cov(sysm,dim)
global ix2
nx2 = dim.x2;
Sigmau = sysm.R(ix2,:)*sysm.R(ix2,:)';
G0v = (eye(nx2^2)-kron(sysm.hl(ix2,ix2),sysm.hl(ix2,ix2)))\Sigmau(1:nx2^2)';
G0  = zeros(nx2,nx2);
G0(1:nx2^2) = G0v;
end

function [y,ind]= resmpl_mul(x,w)
% Y=RESMPL(X,W)
% resamples from X with weights W, with replacement
% based on multinomial sampling
N = size(x,2);
ind = randsample(1:N,N,true,w);
y = x(:,ind);
end